@@ -46,6 +46,8 @@ const ManagerProxy = Gio.DBusProxy.makeProxyWrapper(ManagerInterface);
 const ClientInterface = '<node> \
 <interface name="org.freedesktop.GeoClue2.Client"> \
     <property name="Location" type="o" access="read"/> \
+    <property name="DesktopId" type="s" access="readwrite"/> \
+    <property name="RequestedAccuracyLevel" type="u" access="readwrite"/> \
     <property name="DistanceThreshold" type="u" access="readwrite"/> \
     <method name="Start"/> \
     <method name="Stop"/> \
@@ -57,6 +59,14 @@ const ClientInterface = '<node> \
 </node>';
 const ClientProxy = Gio.DBusProxy.makeProxyWrapper(ClientInterface);
 
+const AccuracyLevel = {
+    COUNTRY: 1,
+    CITY: 4,
+    NEIGHBORHOOD: 5,
+    STREET: 6,
+    EXACT: 8,
+};
+
 const LocationInterface = '<node> \
 <interface name="org.freedesktop.GeoClue2.Location"> \
     <property name="Latitude" type="d" access="read"/> \
@@ -126,6 +136,8 @@ const Geoclue = new Lang.Class({
         this._clientProxy = new ClientProxy(Gio.DBus.system,
                                             "org.freedesktop.GeoClue2",
                                             clientPath);
+        this._clientProxy.DesktopId = "gnome-maps";
+        this._clientProxy.RequestedAccuracyLevel = AccuracyLevel.EXACT;
 
         if (!this.userSetLocation)
             this.findLocation();
